'use strict';

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _flowBin = require('flow-bin');

var _flowBin2 = _interopRequireDefault(_flowBin);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _child_process = require('child_process');

var _child_process2 = _interopRequireDefault(_child_process);

var _filter = require('./filter');

var _filter2 = _interopRequireDefault(_filter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Wrap critical Flow exception into default Error json format
 */
/**
 * Run Flow and collect errors in JSON format
 * Reference https://github.com/facebook/nuclide/blob/master/pkg/nuclide-flow-rpc/lib/FlowRoot.js
 */
function fatalError(stderr) {
  return {
    errors: [{
      message: [{
        path: '',
        code: 0,
        line: 0,
        start: 0,
        descr: stderr
      }]
    }]
  };
}

function getFlowBin() {
  return process.env.FLOW_BIN || _flowBin2.default;
}

function executeFlow() {
  var args = ['--json'];

  var _childProcess$spawnSy = _child_process2.default.spawnSync(getFlowBin(), args);

  var stdout = _childProcess$spawnSy.stdout;

  //
  // This serves as a temporary HACK to prevent 32 bit OS's from failing. Flow does not
  // support 32 bit OS's at the moment.
  // This pretends as if there are now flow errors
  //
  // Ideally, there would be a preinstall npm event to check if the user is on a 32 bit OS
  //

  if (!stdout) {
    return true;
  }

  var stringifiedStdout = stdout.toString();
  var parsedJSONArray = void 0;

  try {
    parsedJSONArray = JSON.parse(stringifiedStdout);
  } catch (e) {
    parsedJSONArray = fatalError(stringifiedStdout);
  }

  // Loop through errors in the file
  var output = parsedJSONArray.errors.map(function (error) {
    return error.message.map(function (message, i, whole) {
      if (message.type === 'Comment' || !message.loc) {
        return false;
      }

      var comments = whole.find(function (_) {
        return _.type === 'Comment';
      });
      var messageDescr = (comments ? comments.descr : '') + ' ' + message.descr;

      if (process.env.DEBUG_FLOWTYPE_ERRRORS === 'true') {
        return {
          message: messageDescr,
          path: message.path,
          start: message.loc.start.line,
          end: message.loc.end.line,
          parsedJSONArray: parsedJSONArray
        };
      }

      return {
        message: messageDescr,
        path: message.path,
        start: message.loc.start.line,
        end: message.loc.end.line
      };
    });
  }).reduce(function (p, c) {
    return p.concat(c);
  }, []);

  return output.length ? (0, _filter2.default)(output) : true;
}

function Flow() {
  var filepath = arguments.length <= 0 || arguments[0] === undefined ? './' : arguments[0];

  return executeFlow(_path2.default.normalize(filepath), {});
}

process.stdout.write((0, _stringify2.default)(Flow()));

module.exports = Flow;